﻿using System;
using System.Net;
using System.Windows.Forms;
using Codeplex.Data;
using ExcelDna.Integration;
using ExcelDna.Integration.Rtd;

    public static class SavePower
    {
        [ExcelCommand(MenuName = "&TestOnRecalc", MenuText = "&Enable Beep")]
        public static void EnableBeep()
        {
            XlCall.Excel(XlCall.xlcOnRecalc, null, "Beep");
        }
        public static void Beep()
        {
            Console.Beep();
        }
        public static object AccessTime()
        {
            object x = XlCall.RTD("Tepco.TepcoServer", null, "");
            return Data.access_time;
        }
        public static object Span()
        {
            object x = XlCall.RTD("Tepco.TepcoServer", null, "");
            return "時刻：" + Data.year + "/" + Data.month + "/" + Data.day + "/" +
                Data.hour + "時台 (" + Data.hour + "時～" + (int.Parse(Data.hour) + 1).ToString() + "時)";
        }
        public static object Saving()
        {
            object x = XlCall.RTD("Tepco.TepcoServer", null, "");
            if (Data.saving == "False")
            {
                return "計画停電：実施無し";
            }
            return "計画停電：実施中";
        }
        public static object Percentage()
        {
            object x = XlCall.RTD("Tepco.TepcoServer", null, "");
            double usage = double.Parse(Data.usage);
            double capacity = double.Parse(Data.capacity);
            return "使用率：" + (usage / capacity).ToString("P");
        }
        public static object Usage()
        {
            object x = XlCall.RTD("Tepco.TepcoServer", null, "");
            return "消費電力：" + Data.usage + "万Kw";
        }
        public static object Capacity()
        {
            object x = XlCall.RTD("Tepco.TepcoServer", null, "");
            return "供給可能最大電力：" + Data.capacity + "万Kw";
        }
        public static object PureUsage()
        {
            object x = XlCall.RTD("Tepco.TepcoServer", null, "");
            return Data.usage;
        }
        public static object PureCapacity()
        {
            object x = XlCall.RTD("Tepco.TepcoServer", null, "");
            return Data.capacity;
        }
    }

    public class TepcoServer : IRtdServer
    {
        private IRTDUpdateEvent _callback;
        private Timer _timer;
        private int _topicId;
        private const int INTERVAL = 300000;

        #region IRtdServer Members
        public object ConnectData(int topicId, ref Array Strings, ref bool GetNewValues)
        {
            _topicId = topicId;
            _timer.Start();
            return 1;
        }
        public void DisconnectData(int topicId)
        {
            _timer.Stop();
        }
        public int Heartbeat()
        {
            return 1;
        }
        public Array RefreshData(ref int topicCount)
        {
            Data.Get();
            object[,] results = new object[2, 1];
            results[0, 0] = _topicId;
            results[1, 0] = GetTime();
            topicCount = 1;
            _timer.Start();
            return results;
        }
        public int ServerStart(IRTDUpdateEvent CallbackObject)
        {
            Data.Get();
            _callback = CallbackObject;
            _timer = new Timer();
            _timer.Tick += Callback;
            _timer.Interval = INTERVAL;
            return 1;
        }
        public void ServerTerminate()
        {
            if (_timer != null)
            {
                _timer.Dispose();
                _timer = null;
            }
        }
        private string GetTime()
        {
            return DateTime.Now.ToString("HH:mm:ss.fff");
        }
        private void Callback(object sender, EventArgs e)
        {
            _timer.Stop();
            _callback.UpdateNotify();
        }
        #endregion
    }

    public static class Data
    {
        public static string access_time;
        public static string year;
        public static string month;
        public static string day;
        public static string hour;
        public static string capacity;
        public static string usage;
        public static string saving;
        [ExcelFunctionAttribute(IsHidden = true)]
        public static bool Get()
        {
            access_time = DateTime.Now.ToString();
            try
            {
                var json_data = new WebClient().DownloadString(@"http://tepco-usage-api.appspot.com/latest.json");
                var status = DynamicJson.Parse(json_data);
                if (status.year != null)
                {
                    year = status.year.ToString();
                }
                if (status.month != null)
                {
                    month = status.month.ToString();
                }
                if (status.day != null)
                {
                    day = status.day.ToString();
                }
                if (status.hour != null)
                {
                    hour = status.hour.ToString();
                }
                if (status.capacity != null)
                {
                    capacity = status.capacity.ToString();
                }
                if (status.usage != null)
                {
                    usage = status.usage.ToString();
                }
                if (status.saving != null)
                {
                    saving = status.saving.ToString();
                }
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }
    }





